"""High-level SDK for Better Notion SDK.

This module provides enriched abstractions and caching
on top of the low-level API.
"""

from better_notion._sdk.client import NotionClient

__all__ = ["NotionClient"]
